@@ -38,8 +38,10 @@
 
 #include "containment.h"
 #include "pluginloader.h"
+#include "packagestructure.h"
 #include "private/applet_p.h"
 #include "private/containment_p.h"
+#include "private/package_p.h"
 #include "private/timetracker.h"
 
 using namespace Plasma;
@@ -71,15 +73,28 @@ Corona::~Corona()
 
 Plasma::Package Corona::package() const
 {
-    return d->package;
+    Package p;
+    p.d->internalPackage = new KPackage::Package(d->package);
+    return p;
 }
 
 void Corona::setPackage(const Plasma::Package &package)
 {
-    d->package = package;
+    setKPackage(*package.d->internalPackage);
     emit packageChanged(package);
 }
 
+KPackage::Package Corona::kPackage() const
+{
+    return d->package;
+}
+
+void Corona::setKPackage(const KPackage::Package &package)
+{
+    d->package = package;
+    emit kPackageChanged(package);
+}
+
 void Corona::saveLayout(const QString &configName) const
 {
     KSharedConfigPtr c;
@@ -477,7 +492,11 @@ Containment *CoronaPrivate::addContainment(const QString &name, const QVariantLi
         conf.deleteGroup();
     }
 
-    containments.append(containment);
+    //make sure the containments are sorted by id
+    auto position = std::lower_bound(containments.begin(), containments.end(), containment, [](Plasma::Containment *c1,  Plasma::Containment *c2) {
+        return c1->id() < c2->id();
+    });
+    containments.insert(position, containment);
 
     QObject::connect(containment, SIGNAL(destroyed(QObject*)),
                      q, SLOT(containmentDestroyed(QObject*)));
@@ -514,8 +533,10 @@ QList<Plasma::Containment *> CoronaPrivate::importLayout(const KConfigGroup &con
     }
 
     KConfigGroup containmentsGroup(&conf, "Containments");
+    QStringList groups = containmentsGroup.groupList();
+    qSort(groups.begin(), groups.end());
 
-    foreach (const QString &group, containmentsGroup.groupList()) {
+    foreach (const QString &group, groups) {
         KConfigGroup containmentConfig(&containmentsGroup, group);
 
         if (containmentConfig.entryMap().isEmpty()) {
@@ -557,13 +578,12 @@ QList<Plasma::Containment *> CoronaPrivate::importLayout(const KConfigGroup &con
     if (!mergeConfig) {
         containmentsStarting = 0;
         foreach (Containment *containment, containments) {
-            if (!containment->isUiReady() && containment->lastScreen() < q->numScreens()) {
+            if (!containment->isUiReady() && containment->screen() < q->numScreens() && containment->screen() >= 0) {
                 ++containmentsStarting;
                 QObject::connect(containment, &Plasma::Containment::uiReadyChanged, [=](bool ready) {
                     if (!ready) {
                         return;
                     }
-
                     --containmentsStarting;
                     if (containmentsStarting <= 0) {
                         emit q->startupCompleted();
