@@ -35,6 +35,7 @@
 #include <kconfiggroup.h>
 #include <QDebug>
 #include <kfilterdev.h>
+#include <kiconeffect.h>
 
 #include "applet.h"
 #include "package.h"
@@ -128,8 +129,8 @@ bool SharedSvgRenderer::load(
 }
 
 #define QLSEP QLatin1Char('_')
-#define CACHE_ID_WITH_SIZE(size, id, devicePixelRatio) QString::number(int(size.width())) % QLSEP % QString::number(int(size.height())) % QLSEP % id % QLSEP % QString::number(int(devicePixelRatio))
-#define CACHE_ID_NATURAL_SIZE(id, devicePixelRatio) QLatin1Literal("Natural") % QLSEP % id % QLSEP % QString::number(int(devicePixelRatio))
+#define CACHE_ID_WITH_SIZE(size, id, devicePixelRatio) QString::number(int(size.width())) % QLSEP % QString::number(int(size.height())) % QLSEP % id % QLSEP % QLSEP % QString::number(int(devicePixelRatio))
+#define CACHE_ID_NATURAL_SIZE(id, devicePixelRatio) QLatin1Literal("Natural") % QLSEP % id % QLSEP % QLSEP % QString::number(int(devicePixelRatio))
 
 SvgPrivate::SvgPrivate(Svg *svg)
     : q(svg),
@@ -138,6 +139,7 @@ SvgPrivate::SvgPrivate(Svg *svg)
       colorGroup(Plasma::Theme::NormalColorGroup),
       lastModified(0),
       devicePixelRatio(1.0),
+      scaleFactor(1.0),
       multipleImages(false),
       themed(false),
       fromCurrentTheme(false),
@@ -232,13 +234,13 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
     if ((themed && QFile::exists(path)) || QFile::exists(actualPath)) {
         QRectF rect;
 
-        if (cacheAndColorsTheme()->findInRectsCache(path, "_Natural", rect)) {
+        if (cacheAndColorsTheme()->findInRectsCache(path, QString("_Natural_%1").arg(scaleFactor), rect)) {
             naturalSize = rect.size();
         } else {
             createRenderer();
-            naturalSize = renderer->defaultSize() * devicePixelRatio;
+            naturalSize = renderer->defaultSize() * scaleFactor;
             //qDebug() << "natural size for" << path << "from renderer is" << naturalSize;
-            cacheAndColorsTheme()->insertIntoRectsCache(path, "_Natural", QRectF(QPointF(0, 0), naturalSize));
+            cacheAndColorsTheme()->insertIntoRectsCache(path, QString("_Natural_%1").arg(scaleFactor), QRectF(QPointF(0, 0), naturalSize));
             //qDebug() << "natural size for" << path << "from cache is" << naturalSize;
         }
     }
@@ -249,6 +251,7 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
         lastModified = info.lastModified().toTime_t();
     }
 
+    q->resize();
     emit q->imagePathChanged();
 
     return updateNeeded;
@@ -278,7 +281,7 @@ Theme *SvgPrivate::cacheAndColorsTheme()
     }
 }
 
-QPixmap SvgPrivate::findInCache(const QString &elementId, const QSizeF &s)
+QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSizeF &s)
 {
     QSize size;
     QString actualElementId;
@@ -286,7 +289,7 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, const QSizeF &s)
     if (elementsWithSizeHints.isEmpty()) {
         // Fetch all size hinted element ids from the theme's rect cache
         // and store them locally.
-        QRegExp sizeHintedKeyExpr(CACHE_ID_NATURAL_SIZE("(\\d+)-(\\d+)-(.+)", devicePixelRatio));
+        QRegExp sizeHintedKeyExpr(CACHE_ID_NATURAL_SIZE("(\\d+)-(\\d+)-(.+)", ratio));
 
         foreach (const QString &key, cacheAndColorsTheme()->listCachedRectKeys(path)) {
             if (sizeHintedKeyExpr.exactMatch(key)) {
@@ -316,7 +319,7 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, const QSizeF &s)
 
             Q_FOREACH (const QSize &hint, elementSizeHints) {
 
-                if (hint.width() >= s.width() && hint.height() >= s.height() &&
+                if (hint.width() >= s.width() * ratio && hint.height() >= s.height() * ratio &&
                         (!bestFit.isValid() ||
                          (bestFit.width() * bestFit.height()) > (hint.width() * hint.height()))) {
                     bestFit = hint;
@@ -335,9 +338,9 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, const QSizeF &s)
     }
 
     if (elementId.isEmpty() || (multipleImages && s.isValid())) {
-        size = s.toSize();
+        size = s.toSize() * ratio;
     } else {
-        size = elementRect(actualElementId).size().toSize();
+        size = elementRect(actualElementId).size().toSize() * ratio;
     }
 
     if (size.isEmpty()) {
@@ -354,6 +357,7 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, const QSizeF &s)
 
     QPixmap p;
     if (cacheRendering && cacheAndColorsTheme()->findInCache(id, p, lastModified)) {
+        p.setDevicePixelRatio(ratio);
         //qDebug() << "found cached version of " << id << p.size();
         return p;
     }
@@ -381,6 +385,14 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, const QSizeF &s)
     }
 
     renderPainter.end();
+    p.setDevicePixelRatio(ratio);
+
+    // Apply current color scheme if the svg asks for it
+    if (applyColors) {
+        QImage itmp = p.toImage();
+        KIconEffect::colorize(itmp, cacheAndColorsTheme()->color(Theme::BackgroundColor), 1.0);
+        p = p.fromImage(itmp);
+    }
 
     if (cacheRendering) {
         cacheAndColorsTheme()->insertIntoCache(id, p, QString::number((qint64)q, 16) % QLSEP % actualElementId);
@@ -401,8 +413,8 @@ void SvgPrivate::createRenderer()
         //FIXME: this maybe could be more efficient if we knew if the package was empty, e.g. for
         //C++; however, I'm not sure this has any real world runtime impact. something to measure
         //for.
-        if (applet && applet->package().isValid()) {
-            const Package package = applet->package();
+        if (applet && applet->kPackage().isValid()) {
+            const KPackage::Package package = applet->kPackage();
             path = package.filePath("images", themePath + ".svg");
 
             if (path.isEmpty()) {
@@ -530,12 +542,13 @@ QRectF SvgPrivate::findAndCacheElementRect(const QString &elementId)
     QRectF elementRect = renderer->elementExists(elementId) ?
                          renderer->matrixForElement(elementId).map(renderer->boundsOnElement(elementId)).boundingRect() :
                          QRectF();
-    naturalSize = renderer->defaultSize();
-    qreal dx = size.width() / naturalSize.width();
-    qreal dy = size.height() / naturalSize.height();
+    naturalSize = renderer->defaultSize() * scaleFactor;
+
+    qreal dx = size.width() / renderer->defaultSize().width();
+    qreal dy = size.height() / renderer->defaultSize().height();
 
     elementRect = QRectF(elementRect.x() * dx, elementRect.y() * dy,
-                         elementRect.width() * dx * devicePixelRatio, elementRect.height() * dy * devicePixelRatio);
+                         elementRect.width() * dx, elementRect.height() * dy);
 
     cacheAndColorsTheme()->insertIntoRectsCache(path, id, elementRect);
 
@@ -550,7 +563,10 @@ QMatrix SvgPrivate::matrixForElement(const QString &elementId)
 
 void SvgPrivate::checkColorHints()
 {
-    if (elementRect("current-color-scheme").isValid()) {
+    if (elementRect("hint-apply-color-scheme").isValid()) {
+        applyColors = true;
+        usesColors = true;
+    } else if (elementRect("current-color-scheme").isValid()) {
         applyColors = false;
         usesColors = true;
     } else {
@@ -644,6 +660,7 @@ void SvgPrivate::themeChanged()
     themePath.clear();
     eraseRenderer();
     setImagePath(currentPath);
+    q->resize();
 
     //qDebug() << themePath << ">>>>>>>>>>>>>>>>>> theme changed";
     emit q->repaintNeeded();
@@ -657,6 +674,17 @@ void SvgPrivate::colorsChanged()
 
     eraseRenderer();
     //qDebug() << "repaint needed from colorsChanged";
+
+    // in the case the theme follows the desktop settings, refetch the colorschemes
+    // and discard the svg pixmap cache
+    if (!theme.data()->d->colors) {
+        KSharedConfig::openConfig()->reparseConfiguration();
+        theme.data()->d->colorScheme = KColorScheme(QPalette::Active, KColorScheme::Window);
+        theme.data()->d->buttonColorScheme = KColorScheme(QPalette::Active, KColorScheme::Button);
+        theme.data()->d->viewColorScheme = KColorScheme(QPalette::Active, KColorScheme::View);
+        theme.data()->d->discardCache(PixmapCache | SvgElementsCache);
+    }
+
     emit q->repaintNeeded();
 }
 
@@ -683,9 +711,13 @@ void Svg::setDevicePixelRatio(qreal ratio)
         return;
     }
 
+    if (FrameSvg *f = qobject_cast<FrameSvg *>(this)) {
+        f->clearCache();
+    }
+
     d->devicePixelRatio = floor(ratio);
+
     emit repaintNeeded();
-    emit sizeChanged();
 }
 
 qreal Svg::devicePixelRatio()
@@ -693,6 +725,38 @@ qreal Svg::devicePixelRatio()
     return d->devicePixelRatio;
 }
 
+
+void Svg::setScaleFactor(qreal ratio)
+{
+    //be completely integer for now
+    //devicepixelratio is always set integer in the svg, so needs at least 192dpi to double up.
+    //(it needs to be integer to have lines contained inside a svg piece to keep being pixel aligned)
+    if (floor(d->scaleFactor) == floor(ratio)) {
+        return;
+    }
+
+    d->scaleFactor = floor(ratio);
+    //not resize() because we want to do it unconditionally
+    QRectF rect;
+
+    if (d->cacheAndColorsTheme()->findInRectsCache(d->path, QString("_Natural_%1").arg(d->scaleFactor), rect)) {
+        d->naturalSize = rect.size();
+    } else {
+        d->createRenderer();
+        d->naturalSize = d->renderer->defaultSize() * d->scaleFactor;
+    }
+
+    d->size = d->naturalSize;
+
+    emit repaintNeeded();
+    emit sizeChanged();
+}
+
+qreal Svg::scaleFactor() const
+{
+    return d->scaleFactor;
+}
+
 void Svg::setColorGroup(Plasma::Theme::ColorGroup group)
 {
     if (d->colorGroup == group) {
@@ -713,28 +777,30 @@ Plasma::Theme::ColorGroup Svg::colorGroup() const
 QPixmap Svg::pixmap(const QString &elementID)
 {
     if (elementID.isNull() || d->multipleImages) {
-        return d->findInCache(elementID, size());
+        return d->findInCache(elementID, d->devicePixelRatio, size());
     } else {
-        return d->findInCache(elementID);
+        return d->findInCache(elementID, d->devicePixelRatio);
     }
 }
 
 QImage Svg::image(const QSize &size, const QString &elementID)
 {
-    QPixmap pix(d->findInCache(elementID, size));
+    QPixmap pix(d->findInCache(elementID, d->devicePixelRatio, size));
     return pix.toImage();
 }
 
 void Svg::paint(QPainter *painter, const QPointF &point, const QString &elementID)
 {
-    QPixmap pix((elementID.isNull() || d->multipleImages) ? d->findInCache(elementID, size()) :
-                d->findInCache(elementID));
+    Q_ASSERT(painter->device());
+    const int ratio = painter->device()->devicePixelRatio();
+    QPixmap pix((elementID.isNull() || d->multipleImages) ? d->findInCache(elementID, ratio, size()) :
+                d->findInCache(elementID, ratio));
 
     if (pix.isNull()) {
         return;
     }
 
-    painter->drawPixmap(QRectF(point, pix.size()), pix, QRectF(QPointF(0, 0), pix.size()));
+    painter->drawPixmap(QRectF(point, size()), pix, QRectF(QPointF(0, 0), pix.size()));
 }
 
 void Svg::paint(QPainter *painter, int x, int y, const QString &elementID)
@@ -744,13 +810,18 @@ void Svg::paint(QPainter *painter, int x, int y, const QString &elementID)
 
 void Svg::paint(QPainter *painter, const QRectF &rect, const QString &elementID)
 {
-    QPixmap pix(d->findInCache(elementID, rect.size()));
-    painter->drawPixmap(QRectF(rect.topLeft(), pix.size()), pix, QRectF(QPointF(0, 0), pix.size()));
+    Q_ASSERT(painter->device());
+    const int ratio = painter->device()->devicePixelRatio();
+    QPixmap pix(d->findInCache(elementID, ratio, rect.size()));
+
+    painter->drawPixmap(QRectF(rect.topLeft(), rect.size()), pix, QRectF(QPointF(0, 0), pix.size()));
 }
 
 void Svg::paint(QPainter *painter, int x, int y, int width, int height, const QString &elementID)
 {
-    QPixmap pix(d->findInCache(elementID, QSizeF(width, height)));
+    Q_ASSERT(painter->device());
+    const int ratio = painter->device()->devicePixelRatio();
+    QPixmap pix(d->findInCache(elementID, ratio, QSizeF(width, height)));
     painter->drawPixmap(x, y, pix, 0, 0, pix.size().width(), pix.size().height());
 }
 
@@ -819,7 +890,7 @@ bool Svg::isValid() const
 
     //try very hard to avoid creation of a parser
     QRectF rect;
-    if (d->cacheAndColorsTheme()->findInRectsCache(d->path, "_Natural", rect)) {
+    if (d->cacheAndColorsTheme()->findInRectsCache(d->path, QString("_Natural_%1").arg(d->scaleFactor), rect)) {
         return true;
     }
 
